@@ -41,7 +41,7 @@ private:
     GdkRectangle rect_;
     double scrollDelta_ = 0;
     const bool isWayland_ = false;
-    GdkRectangle lastRect_;
+    GdkRectangle lastRect_ = {0, 0, 0, 0};
 };
 
 } // namespace fcitx::gtk
